@@ -33,11 +33,15 @@
 #include "nautilus-file-utilities.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-macros.h>
+#include <gconf/gconf-client.h>
+#include <gconf/gconf-value.h>
 #include "nautilus-desktop-directory.h"
-#include <gtk/gtksignal.h>
-#include <libgnome/gnome-i18n.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <string.h>
 
+#define NAUTILUS_DESKTOP_METADATA_GCONF_PATH "/apps/nautilus/desktop-metadata"
+
 struct NautilusDesktopDirectoryFileDetails {
 	NautilusDesktopDirectory *desktop_directory;
 
@@ -69,7 +73,7 @@ typedef struct {
 
 
 static void nautilus_desktop_directory_file_init       (gpointer   object,
-						  gpointer   klass);
+							gpointer   klass);
 static void nautilus_desktop_directory_file_class_init (gpointer   klass);
 
 EEL_CLASS_BOILERPLATE (NautilusDesktopDirectoryFile,
@@ -370,12 +374,6 @@ desktop_directory_file_check_if_ready (NautilusFile *file,
 					      delegated_attributes);
 }
 
-static GnomeVFSFileType
-desktop_directory_file_get_file_type (NautilusFile *file)
-{
-	return GNOME_VFS_FILE_TYPE_DIRECTORY;
-}			      
-
 static gboolean
 desktop_directory_file_get_item_count (NautilusFile *file, 
 				       guint *count,
@@ -402,7 +400,7 @@ desktop_directory_file_get_deep_counts (NautilusFile *file,
 					guint *directory_count,
 					guint *file_count,
 					guint *unreadable_directory_count,
-					GnomeVFSFileSize *total_size)
+					goffset *total_size)
 {
 	NautilusDesktopDirectoryFile *desktop_file;
 	NautilusRequestStatus status;
@@ -454,6 +452,156 @@ monitor_destroy (gpointer data)
 	g_free (monitor);
 }
 
+static char *
+get_metadata_gconf_path (const char *name,
+			 const char *key)
+{
+	char *res, *escaped_name;
+
+	escaped_name = gconf_escape_key (name, -1);
+	res = g_build_filename (NAUTILUS_DESKTOP_METADATA_GCONF_PATH, escaped_name, key, NULL);
+	g_free (escaped_name);
+
+	return res;
+}
+
+void
+nautilus_desktop_set_metadata_string (NautilusFile *file,
+				      const char *name,
+				      const char *key,
+				      const char *string)
+{
+	GConfClient *client;
+	char *gconf_key;
+
+	client = gconf_client_get_default ();
+	gconf_key = get_metadata_gconf_path (name, key);
+
+	if (string) {
+		gconf_client_set_string (client, gconf_key, string, NULL);
+	} else {
+		gconf_client_unset (client, gconf_key, NULL);
+	}
+
+	g_free (gconf_key);
+	g_object_unref (client);
+
+	if (nautilus_desktop_update_metadata_from_gconf (file, name)) {
+		nautilus_file_changed (file);
+	}
+}
+
+void
+nautilus_desktop_set_metadata_stringv (NautilusFile *file,
+				       const char *name,
+				       const char *key,
+				       char **stringv)
+{
+	GConfClient *client;
+	char *gconf_key;
+	GSList *list;
+	int i;
+
+	client = gconf_client_get_default ();
+	gconf_key = get_metadata_gconf_path (name, key);
+
+	list = NULL;
+	for (i = 0; stringv[i] != NULL; i++) {
+		list = g_slist_prepend (list, stringv[i]);
+	}
+	list = g_slist_reverse (list);
+
+	gconf_client_set_list (client, gconf_key,
+			       GCONF_VALUE_STRING,
+			       list, NULL);
+
+	g_slist_free (list);
+	g_free (gconf_key);
+	g_object_unref (client);
+
+	if (nautilus_desktop_update_metadata_from_gconf (file, name)) {
+		nautilus_file_changed (file);
+	}
+}
+
+gboolean
+nautilus_desktop_update_metadata_from_gconf (NautilusFile *file,
+					     const char *name)
+{
+	GConfClient *client;
+	GSList *entries, *l;
+	char *dir;
+	const char *key;
+	GConfEntry *entry;
+	GConfValue *value;
+	GFileInfo *info;
+	gboolean changed;
+	char *gio_key;
+	GSList *value_list;
+	char **strv;
+	int i;
+
+	client = gconf_client_get_default ();
+
+	dir = get_metadata_gconf_path (name, NULL);
+	entries = gconf_client_all_entries (client, dir, NULL);
+	g_free (dir);
+
+	info = g_file_info_new ();
+
+	for (l = entries; l != NULL; l = l->next) {
+		entry = l->data;
+
+		key = gconf_entry_get_key (entry);
+		value = gconf_entry_get_value (entry);
+
+		key = strrchr (key, '/') + 1;
+
+		gio_key = g_strconcat ("metadata::", key, NULL);
+		if (value->type == GCONF_VALUE_STRING) {
+			g_file_info_set_attribute_string (info, gio_key,
+							  gconf_value_get_string (value));
+		} else if (value->type == GCONF_VALUE_LIST &&
+			   gconf_value_get_list_type (value) == GCONF_VALUE_STRING) {
+			value_list = gconf_value_get_list (value);
+			strv = g_new (char *, g_slist_length (value_list) + 1);
+			for (i = 0; value_list != NULL; i++, value_list = value_list->next) {
+				strv[i] = l->data;
+			}
+			strv[i] = NULL;
+			g_file_info_set_attribute_stringv (info, gio_key, strv);
+			g_free (strv);
+		}
+
+		g_free (gio_key);
+
+		gconf_entry_unref (entry);
+	}
+
+	changed = nautilus_file_update_metadata_from_info (file, info);
+
+	g_object_unref (info);
+	g_object_unref (client);
+
+	return changed;
+}
+
+static void
+nautilus_desktop_directory_file_set_metadata (NautilusFile           *file,
+					      const char             *key,
+					      const char             *value)
+{
+	nautilus_desktop_set_metadata_string (file, "directory", key, value);
+}
+
+static void
+nautilus_desktop_directory_file_set_metadata_as_list (NautilusFile           *file,
+						      const char             *key,
+						      char                  **value)
+{
+	nautilus_desktop_set_metadata_stringv (file, "directory", key, value);
+}
+
 static void
 nautilus_desktop_directory_file_init (gpointer object, gpointer klass)
 {
@@ -464,7 +612,7 @@ nautilus_desktop_directory_file_init (gpointer object, gpointer klass)
 
 	desktop_file = NAUTILUS_DESKTOP_DIRECTORY_FILE (object);
 
-	desktop_directory = NAUTILUS_DESKTOP_DIRECTORY (nautilus_directory_get (EEL_DESKTOP_URI));
+	desktop_directory = NAUTILUS_DESKTOP_DIRECTORY (nautilus_directory_get_by_uri (EEL_DESKTOP_URI));
 
 	desktop_file->details = g_new0 (NautilusDesktopDirectoryFileDetails, 1);
 	desktop_file->details->desktop_directory = desktop_directory;
@@ -473,13 +621,15 @@ nautilus_desktop_directory_file_init (gpointer object, gpointer klass)
 		(desktop_callback_hash, desktop_callback_equal);
 	desktop_file->details->monitors = g_hash_table_new_full (NULL, NULL,
 								 NULL, monitor_destroy);
-	
+
 	real_dir = nautilus_desktop_directory_get_real_directory (desktop_directory);
 	real_dir_file = nautilus_directory_get_corresponding_file (real_dir);
 	nautilus_directory_unref (real_dir);
 	
 	desktop_file->details->real_dir_file = real_dir_file;
-	
+
+	nautilus_desktop_update_metadata_from_gconf (NAUTILUS_FILE (desktop_file), "directory");
+
 	g_signal_connect_object (real_dir_file, "changed",
 				 G_CALLBACK (real_file_changed_callback), desktop_file, 0);
 }
@@ -537,14 +687,17 @@ nautilus_desktop_directory_file_class_init (gpointer klass)
 	
 	object_class->finalize = desktop_finalize;
 
+	file_class->default_file_type = G_FILE_TYPE_DIRECTORY;
+	
 	file_class->monitor_add = desktop_directory_file_monitor_add;
 	file_class->monitor_remove = desktop_directory_file_monitor_remove;
 	file_class->call_when_ready = desktop_directory_file_call_when_ready;
 	file_class->cancel_call_when_ready = desktop_directory_file_cancel_call_when_ready;
 	file_class->check_if_ready = desktop_directory_file_check_if_ready;
-	file_class->get_file_type = desktop_directory_file_get_file_type;
 	file_class->get_item_count = desktop_directory_file_get_item_count;
 	file_class->get_deep_counts = desktop_directory_file_get_deep_counts;
 	file_class->get_date = desktop_directory_file_get_date;
 	file_class->get_where_string = desktop_directory_file_get_where_string;
+	file_class->set_metadata = nautilus_desktop_directory_file_set_metadata;
+	file_class->set_metadata_as_list = nautilus_desktop_directory_file_set_metadata_as_list;
 }
