@@ -55,10 +55,12 @@
 #include "libnautilus-private/nautilus-file-operations.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-manage-views.h"
+#include <unistd.h>
 #include <libxml/xmlsave.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
+#include <dbus/dbus-glib.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
@@ -94,6 +96,8 @@ enum
 
 #define START_STATE_CONFIG "start-state"
 
+#define NAUTILUS_ACCEL_MAP_SAVE_DELAY 30
+
 /* Keeps track of all the desktop windows. */
 static GList *nautilus_application_desktop_windows;
 
@@ -103,6 +107,9 @@ static GList *nautilus_application_window_list;
 /* Keeps track of all the object windows */
 static GList *nautilus_application_spatial_window_list;
 
+/* The saving of the accelerator map was requested  */
+static gboolean save_of_accel_map_requested = FALSE;
+
 static void     desktop_changed_callback          (gpointer                  user_data);
 static void     desktop_location_changed_callback (gpointer                  user_data);
 static void     mount_removed_callback            (GVolumeMonitor            *monitor,
@@ -122,6 +129,10 @@ static void     drive_listen_for_eject_button      (GDrive *drive,
 static gboolean is_kdesktop_present               (void);
 static void     nautilus_application_load_session     (NautilusApplication *application); 
 static char *   nautilus_application_get_session_data (void);
+static void     ck_session_active_changed_cb (DBusGProxy *proxy,
+		                              gboolean is_active,
+                		              void *user_data);
+
 
 G_DEFINE_TYPE (NautilusApplication, nautilus_application, G_TYPE_OBJECT);
 
@@ -182,7 +193,13 @@ _unique_message_data_get_geometry_and_uris (UniqueMessageData *message_data,
 	  result = g_uri_list_extract_uris (uris);
 	  g_free (text);
   }
-  
+
+  /* if the string is empty, make it NULL */
+  if (*geometry && strlen (*geometry) == 0) {
+	  g_free (*geometry);
+	  *geometry = NULL;
+  }
+
   return result;
 }
 
@@ -310,6 +327,10 @@ nautilus_application_init (NautilusApplication *application)
 
 	/* register property pages */
 	nautilus_image_properties_page_register ();
+
+	/* initialize search path for custom icons */
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
+					   NAUTILUS_DATADIR G_DIR_SEPARATOR_S "icons");
 }
 
 NautilusApplication *
@@ -341,6 +362,13 @@ nautilus_application_finalize (GObject *object)
 		application->automount_idle_id = 0;
 	}
 
+	if (application->ck_session_proxy != NULL) {
+		dbus_g_proxy_disconnect_signal (application->ck_session_proxy, "ActiveChanged",
+						G_CALLBACK (ck_session_active_changed_cb), NULL);
+		g_object_unref (application->ck_session_proxy);
+		application->ck_session_proxy = NULL;
+	}
+
         G_OBJECT_CLASS (nautilus_application_parent_class)->finalize (object);
 }
 
@@ -454,10 +482,169 @@ automount_all_volumes_idle_cb (gpointer data)
 }
 
 static void
-finish_startup (NautilusApplication *application)
+mark_desktop_files_trusted (void)
+{
+	char *do_once_file;
+	GFile *f, *c;
+	GFileEnumerator *e;
+	GFileInfo *info;
+	const char *name;
+	int fd;
+	
+	do_once_file = g_build_filename (g_get_user_data_dir (),
+					 ".converted-launchers", NULL);
+
+	if (g_file_test (do_once_file, G_FILE_TEST_EXISTS)) {
+		goto out;
+	}
+
+	f = nautilus_get_desktop_location ();
+	e = g_file_enumerate_children (f,
+				       G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+				       G_FILE_ATTRIBUTE_STANDARD_NAME ","
+				       G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE
+				       ,
+				       G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+				       NULL, NULL);
+	if (e == NULL) {
+		goto out2;
+	}
+	
+	while ((info = g_file_enumerator_next_file (e, NULL, NULL)) != NULL) {
+		name = g_file_info_get_name (info);
+		
+		if (g_str_has_suffix (name, ".desktop") &&
+		    !g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE)) {
+			c = g_file_get_child (f, name);
+			nautilus_file_mark_desktop_file_trusted (c,
+								 NULL, FALSE,
+								 NULL, NULL);
+			g_object_unref (c);
+		}
+		g_object_unref (info);
+	}
+	
+	g_object_unref (e);
+ out2:
+	fd = g_creat (do_once_file, 0666);
+	close (fd);
+	
+	g_object_unref (f);
+ out:	
+	g_free (do_once_file);
+}
+
+#define CK_NAME "org.freedesktop.ConsoleKit"
+#define CK_PATH "/org/freedesktop/ConsoleKit"
+
+static void
+ck_session_active_changed_cb (DBusGProxy *proxy,
+			      gboolean is_active,
+			      void *user_data)
+{
+	NautilusApplication *application = user_data;
+
+	application->session_is_active = is_active;
+}
+
+static void
+ck_call_is_active_cb (DBusGProxy *proxy,
+		      DBusGProxyCall *call_id,
+		      void *user_data)
+{
+	gboolean res, is_active;
+	NautilusApplication *application;
+
+	application = user_data;
+
+	res = dbus_g_proxy_end_call (proxy, call_id, NULL,
+				     G_TYPE_BOOLEAN, &is_active,
+				     G_TYPE_INVALID);
+	if (!res) {
+		g_object_unref (proxy);
+
+		application->session_is_active = TRUE;
+		return;
+	}
+
+	application->session_is_active = is_active;
+
+	dbus_g_proxy_add_signal (proxy, "ActiveChanged", G_TYPE_BOOLEAN, G_TYPE_INVALID);
+	dbus_g_proxy_connect_signal (proxy, "ActiveChanged",
+				     G_CALLBACK (ck_session_active_changed_cb), application,
+				     NULL);
+}
+
+static void
+ck_get_current_session_cb (DBusGProxy *proxy,
+			   DBusGProxyCall *call_id,
+			   void *user_data)
+{
+	gboolean res;
+	char *session_id;
+	NautilusApplication *application;
+
+	application = user_data;
+
+	res = dbus_g_proxy_end_call (proxy, call_id, NULL,
+				     DBUS_TYPE_G_OBJECT_PATH, &session_id, G_TYPE_INVALID);
+	if (!res) {
+		g_object_unref (proxy);
+
+		application->session_is_active = TRUE;
+		return;
+	}
+
+	application->ck_session_proxy = dbus_g_proxy_new_from_proxy (proxy, CK_NAME ".Session",
+								     session_id);
+	dbus_g_proxy_begin_call (application->ck_session_proxy, "IsActive", ck_call_is_active_cb,
+				 application, NULL, G_TYPE_INVALID);
+
+	g_object_unref (proxy);
+}
+
+
+static void
+do_initialize_consolekit (NautilusApplication *application)
+{
+	DBusGConnection *conn;
+	DBusGProxy *proxy;
+	GError *error = NULL;
+
+	conn = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+	if (error) {
+		g_error_free (error);
+
+		application->session_is_active = TRUE;
+
+		return;
+	}
+
+	proxy = dbus_g_proxy_new_for_name (conn, CK_NAME, CK_PATH "/Manager",
+ 					   CK_NAME ".Manager");
+	dbus_g_proxy_begin_call (proxy, "GetCurrentSession",
+				 ck_get_current_session_cb, application,
+				 NULL, G_TYPE_INVALID);
+}
+
+static void
+do_upgrades_once (NautilusApplication *application,
+		  gboolean no_desktop)
+{
+	if (!no_desktop) {
+		mark_desktop_files_trusted ();
+	}
+}
+
+
+static void
+finish_startup (NautilusApplication *application,
+		gboolean no_desktop)
 {
 	GList *drives;
 
+	do_upgrades_once (application, no_desktop);
+	
 	/* initialize nautilus modules */
 	nautilus_module_setup ();
 
@@ -467,6 +654,9 @@ finish_startup (NautilusApplication *application)
 	/* Initialize the desktop link monitor singleton */
 	nautilus_desktop_link_monitor_get ();
 
+	/* Initialize the ConsoleKit listener for active session */
+	do_initialize_consolekit (application);
+
 	/* Watch for mounts so we can restore open windows This used
 	 * to be for showing new window on mount, but is not used
 	 * anymore */
@@ -500,7 +690,8 @@ finish_startup (NautilusApplication *application)
 static void
 open_window (NautilusApplication *application,
 	     const char *startup_id,
-	     const char *uri, const char *geometry, gboolean browser_window)
+	     const char *uri, GdkScreen *screen, const char *geometry,
+	     gboolean browser_window)
 {
 	GFile *location;
 	NautilusWindow *window;
@@ -509,7 +700,7 @@ open_window (NautilusApplication *application,
 	    eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
 		window = nautilus_application_create_navigation_window (application,
 									startup_id,
-									gdk_screen_get_default ());
+									screen);
 		if (uri == NULL) {
 			nautilus_window_go_home (window);
 		} else {
@@ -528,7 +719,7 @@ open_window (NautilusApplication *application,
 								      NULL,
 								      startup_id,
 								      location,
-								      gdk_screen_get_default ());
+								      screen);
 		g_object_unref (location);
 	}
 	
@@ -549,6 +740,7 @@ static void
 open_windows (NautilusApplication *application,
 	      const char *startup_id,
 	      char **uris,
+	      GdkScreen *screen,
 	      const char *geometry,
 	      gboolean browser_window)
 {
@@ -556,11 +748,11 @@ open_windows (NautilusApplication *application,
 
 	if (uris == NULL || uris[0] == NULL) {
 		/* Open a window pointing at the default location. */
-		open_window (application, startup_id, NULL, geometry, browser_window);
+		open_window (application, startup_id, NULL, screen, geometry, browser_window);
 	} else {
 		/* Open windows at each requested location. */
 		for (i = 0; uris[i] != NULL; i++) {
-			open_window (application, startup_id, uris[i], geometry, browser_window);
+			open_window (application, startup_id, uris[i], screen, geometry, browser_window);
 		}
 	}
 }
@@ -576,6 +768,7 @@ message_received_cb (UniqueApp         *unique_app,
 	UniqueResponse res;
 	char **uris;
 	char *geometry;
+	GdkScreen *screen;
 	
 	application =  user_data;
 	res = UNIQUE_RESPONSE_OK;
@@ -589,9 +782,11 @@ message_received_cb (UniqueApp         *unique_app,
 	case UNIQUE_OPEN:
 	case COMMAND_OPEN_BROWSER:
 		uris = _unique_message_data_get_geometry_and_uris (message, &geometry);
+		screen = unique_message_data_get_screen (message);
 		open_windows (application,
 			      unique_message_data_get_startup_id (message),
 			      uris,
+			      screen,
 			      geometry,
 			      command == COMMAND_OPEN_BROWSER);
 		g_strfreev (uris);
@@ -611,6 +806,35 @@ message_received_cb (UniqueApp         *unique_app,
 	return res;
 }
 
+gboolean 
+nautilus_application_save_accel_map (gpointer data)
+{
+	if (save_of_accel_map_requested) {
+		char *accel_map_filename;
+	 	accel_map_filename = nautilus_get_accel_map_file ();
+	 	if (accel_map_filename) {
+	 		gtk_accel_map_save (accel_map_filename);
+	 		g_free (accel_map_filename);
+	 	}
+		save_of_accel_map_requested = FALSE;
+	}
+
+	return FALSE;
+}
+
+
+static void 
+queue_accel_map_save_callback (GtkAccelMap *object, gchar *accel_path,
+		guint accel_key, GdkModifierType accel_mods,
+		gpointer user_data)
+{
+	if (!save_of_accel_map_requested) {
+		save_of_accel_map_requested = TRUE;
+		g_timeout_add_seconds (NAUTILUS_ACCEL_MAP_SAVE_DELAY, 
+				nautilus_application_save_accel_map, NULL);
+	}
+}
+
 void
 nautilus_application_startup (NautilusApplication *application,
 			      gboolean kill_shell,
@@ -636,12 +860,19 @@ nautilus_application_startup (NautilusApplication *application,
 			
 		}
 	} else {
+		char *accel_map_filename;
+
 		/* If KDE desktop is running, then force no_desktop */
 		if (is_kdesktop_present ()) {
 			no_desktop = TRUE;
 		}
 		
-		if (!no_desktop && eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP)) {
+		if (!no_desktop &&
+		    !eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP)) {
+			no_desktop = TRUE;
+		}
+			
+		if (!no_desktop) {
 			if (unique_app_is_running (application->unique_app)) {
 				unique_app_send_message (application->unique_app,
 							 COMMAND_START_DESKTOP, NULL);
@@ -651,7 +882,7 @@ nautilus_application_startup (NautilusApplication *application,
 		}
 
 		if (!unique_app_is_running (application->unique_app)) {
-			finish_startup (application);
+			finish_startup (application, no_desktop);
 			g_signal_connect (application->unique_app, "message-received", G_CALLBACK (message_received_cb), application);			
 		}
 		
@@ -684,6 +915,7 @@ nautilus_application_startup (NautilusApplication *application,
 			} else {
 				open_windows (application, NULL,
 					      urls,
+					      gdk_screen_get_default (),
 					      geometry,
 					      browser_window);
 			}
@@ -691,6 +923,14 @@ nautilus_application_startup (NautilusApplication *application,
 
 		/* Load session info if availible */
 		nautilus_application_load_session (application);
+		
+		/* load accelerator map, and register save callback */
+		accel_map_filename = nautilus_get_accel_map_file ();
+		if (accel_map_filename) {
+			gtk_accel_map_load (accel_map_filename);
+			g_free (accel_map_filename);
+		}
+		g_signal_connect (gtk_accel_map_get (), "changed", G_CALLBACK (queue_accel_map_save_callback), NULL);
 	}
 }
 
@@ -991,7 +1231,6 @@ create_window (NautilusApplication *application,
 	/* Must be called after construction finished */
 	nautilus_window_constructed (window);
 
-	unique_app_watch_window (application->unique_app, GTK_WINDOW (window));
 	if (startup_id) {
 		gtk_window_set_startup_id (GTK_WINDOW (window), startup_id);
 	}
@@ -1240,7 +1479,7 @@ drive_eject_cb (GObject *source_object,
 	char *primary;
 	char *name;
 	error = NULL;
-	if (!g_drive_eject_finish (G_DRIVE (source_object), res, &error)) {
+	if (!g_drive_eject_with_operation_finish (G_DRIVE (source_object), res, &error)) {
 		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
 			name = g_drive_get_name (G_DRIVE (source_object));
 			primary = g_strdup_printf (_("Unable to eject %s"), name);
@@ -1258,7 +1497,11 @@ static void
 drive_eject_button_pressed (GDrive *drive,
 			    NautilusApplication *application)
 {
-	g_drive_eject (drive, 0, NULL, drive_eject_cb, NULL);
+	GMountOperation *mount_op;
+
+	mount_op = gtk_mount_operation_new (NULL);
+	g_drive_eject_with_operation (drive, 0, mount_op, NULL, drive_eject_cb, NULL);
+	g_object_unref (mount_op);
 }
 
 static void
@@ -1311,6 +1554,10 @@ mount_added_callback (GVolumeMonitor *monitor,
 {
 	NautilusDirectory *directory;
 	GFile *root;
+
+	if (!application->session_is_active) {
+		return;
+	}
 		
 	root = g_mount_get_root (mount);
 	directory = nautilus_directory_get_existing (root);
@@ -1613,7 +1860,7 @@ nautilus_application_load_session (NautilusApplication *application)
 
 	data = g_key_file_get_string (state_file,
 				      "Nautilus",
-				      "sessiondata",
+				      "documents",
 				      NULL);
 	if (data == NULL) {
 		return;
