@@ -32,19 +32,23 @@
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-preferences.h>
 #include <gtk/gtk.h>
-#include <libgnome/gnome-macros.h>
 #include <glib/gi18n.h>
 #include <libnautilus-private/nautilus-bookmark.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-sidebar-provider.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-signaller.h>
+#include <libnautilus-private/nautilus-window-info.h>
+#include <libnautilus-private/nautilus-window-slot-info.h>
 
 #include "nautilus-history-sidebar.h"
 
-#define NAUTILUS_HISTORY_SIDEBAR_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_HISTORY_SIDEBAR, NautilusHistorySidebarClass))
-#define NAUTILUS_IS_HISTORY_SIDEBAR(obj)         (GTK_CHECK_TYPE ((obj), NAUTILUS_TYPE_HISTORY_SIDEBAR))
-#define NAUTILUS_IS_HISTORY_SIDEBAR_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_HISTORY_SIDEBAR))
+#define NAUTILUS_HISTORY_SIDEBAR_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_HISTORY_SIDEBAR, NautilusHistorySidebarClass))
+#define NAUTILUS_IS_HISTORY_SIDEBAR(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NAUTILUS_TYPE_HISTORY_SIDEBAR))
+#define NAUTILUS_IS_HISTORY_SIDEBAR_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_HISTORY_SIDEBAR))
 
 typedef struct {
 	GtkScrolledWindowClass parent;
@@ -118,7 +122,7 @@ update_history (NautilusHistorySidebar *sidebar)
 
 	selection = GTK_TREE_SELECTION (gtk_tree_view_get_selection (sidebar->tree_view));
 
-	if (gtk_tree_model_get_iter_root (GTK_TREE_MODEL (store), &iter)) {
+	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter)) {
 		gtk_tree_selection_select_iter (selection, &iter);
 	}
 }
@@ -131,19 +135,17 @@ history_changed_callback (GObject *signaller,
 }
 
 static void
-row_activated_callback (GtkTreeView *tree_view,
-			GtkTreePath *path,
-			GtkTreeViewColumn *column,
-			gpointer user_data)
+open_selected_item (NautilusHistorySidebar *sidebar, 
+		    GtkTreePath *path,
+		    NautilusWindowOpenFlags flags)
 {
-	NautilusHistorySidebar *sidebar;
+	NautilusWindowSlotInfo *slot;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	NautilusBookmark *bookmark;
 	GFile *location;
 	
-	sidebar = NAUTILUS_HISTORY_SIDEBAR (user_data);
-	model = gtk_tree_view_get_model (tree_view);
+	model = gtk_tree_view_get_model (sidebar->tree_view);
 	
 	if (!gtk_tree_model_get_iter (model, &iter, path)) {
 		return;
@@ -154,12 +156,54 @@ row_activated_callback (GtkTreeView *tree_view,
 	
 	/* Navigate to the clicked location. */
 	location = nautilus_bookmark_get_location (NAUTILUS_BOOKMARK (bookmark));
-	nautilus_window_info_open_location
-		(sidebar->window, 
-		 location, NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 0, NULL);
+	slot = nautilus_window_info_get_active_slot (sidebar->window);
+	nautilus_window_slot_info_open_location
+		(slot,
+		 location, NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 
+		 flags, NULL);
 	g_object_unref (location);
 }
 
+static void
+row_activated_callback (GtkTreeView *tree_view,
+			GtkTreePath *path,
+			GtkTreeViewColumn *column,
+			gpointer user_data)
+{
+	NautilusHistorySidebar *sidebar;
+	
+	sidebar = NAUTILUS_HISTORY_SIDEBAR (user_data);
+	g_assert (sidebar->tree_view == tree_view);
+
+	open_selected_item (sidebar, path, 0);
+}
+
+static gboolean
+button_press_event_callback (GtkWidget *widget,
+			     GdkEventButton *event,
+			     gpointer user_data)
+{
+	if (event->button == 2 && event->type == GDK_BUTTON_PRESS) {
+		/* Open new tab on middle click. */
+		NautilusHistorySidebar *sidebar;
+		GtkTreePath *path;
+
+		sidebar = NAUTILUS_HISTORY_SIDEBAR (user_data);
+		g_assert (sidebar->tree_view == GTK_TREE_VIEW (widget));
+
+		if (gtk_tree_view_get_path_at_pos (sidebar->tree_view,
+						   event->x, event->y,
+						   &path, NULL, NULL, NULL)) {
+			open_selected_item (sidebar, 
+					    path,
+					    NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB);
+			gtk_tree_path_free (path);
+		}
+	}
+
+	return FALSE;
+}
+
 static void
 update_click_policy (NautilusHistorySidebar *sidebar)
 {
@@ -240,6 +284,9 @@ nautilus_history_sidebar_init (NautilusHistorySidebar *sidebar)
 				 "history_list_changed",
 				 G_CALLBACK (history_changed_callback), sidebar, 0);
 
+	g_signal_connect (tree_view, "button-press-event", 
+			  G_CALLBACK (button_press_event_callback), sidebar);
+
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_CLICK_POLICY,
 				      click_policy_changed_callback,
 				      sidebar);
@@ -337,7 +384,6 @@ nautilus_history_sidebar_create (NautilusSidebarProvider *provider,
 	sidebar = g_object_new (nautilus_history_sidebar_get_type (), NULL);
 	nautilus_history_sidebar_set_parent_window (sidebar, window);
 	g_object_ref (sidebar);
-	gtk_object_sink (GTK_OBJECT (sidebar));
 
 	return NAUTILUS_SIDEBAR (sidebar);
 }
