@@ -46,7 +46,6 @@
 #include <glib/gi18n.h>
 #include <gio/gdesktopappinfo.h>
 #include <libnautilus-private/nautilus-debug-log.h>
-#include <libnautilus-private/nautilus-directory-metafile.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-lib-self-check-functions.h>
 #include <libnautilus-private/nautilus-icon-names.h>
@@ -54,6 +53,9 @@
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -65,10 +67,12 @@
 /* Keeps track of everyone who wants the main event loop kept active */
 static GSList *event_loop_registrants;
 
+static gboolean exit_with_last_window = TRUE;
+
 static gboolean
 is_event_loop_needed (void)
 {
-	return event_loop_registrants != NULL;
+	return event_loop_registrants != NULL || !exit_with_last_window;
 }
 
 static int
@@ -129,6 +133,19 @@ nautilus_main_event_loop_quit (gboolean explicit)
 {
 	if (explicit) {
 		/* Explicit --quit, make sure we don't restart */
+
+		/* To quit all instances, reset exit_with_last_window */
+		exit_with_last_window = TRUE;
+
+		if (event_loop_registrants == NULL) {
+			/* If this is reached, nautilus must run in "daemon" mode
+			 * (i.e. !exit_with_last_window) with no windows open.
+			 * We need to quit_all here because the below loop won't
+			 * trigger a quit.
+			 */
+			eel_gtk_main_quit_all();
+		}
+
 		/* TODO: With the old session we needed to set restart
 		   style to GNOME_RESTART_IF_RUNNING here, but i don't think we need
 		   that now since gnome-session doesn't restart apps except on startup. */
@@ -326,7 +343,7 @@ main (int argc, char *argv[])
 		  N_("Perform a quick set of self-check tests."), NULL },
 #endif
 		{ "version", '\0', 0, G_OPTION_ARG_NONE, &version,
-		  N_("Show the version of the progam."), NULL },
+		  N_("Show the version of the program."), NULL },
 		{ "geometry", 'g', 0, G_OPTION_ARG_STRING, &geometry,
 		  N_("Create the initial window with the given geometry."), N_("GEOMETRY") },
 		{ "no-default-window", 'n', 0, G_OPTION_ARG_NONE, &no_default_window,
@@ -342,6 +359,20 @@ main (int argc, char *argv[])
 		{ NULL }
 	};
 
+#if defined (HAVE_MALLOPT) && defined(M_MMAP_THRESHOLD)
+	/* Nautilus uses lots and lots of small and medium size allocations,
+	 * and then a few large ones for the desktop background. By default
+	 * glibc uses a dynamic treshold for how large allocations should
+	 * be mmaped. Unfortunately this triggers quickly for nautilus when
+	 * it does the desktop background allocations, raising the limit
+	 * such that a lot of temporary large allocations end up on the
+	 * heap and are thus not returned to the OS. To fix this we set
+	 * a hardcoded limit. I don't know what a good value is, but 128K
+	 * was the old glibc static limit, lets use that.
+	 */
+	mallopt (M_MMAP_THRESHOLD, 128 *1024);
+#endif
+	
 	g_thread_init (NULL);
 
 	/* This will be done by gtk+ later, but for now, force it to GNOME */
@@ -374,6 +405,7 @@ main (int argc, char *argv[])
 	browser_window = FALSE;
 
 	g_set_prgname ("nautilus");
+
 	if (g_file_test (DATADIR "/applications/nautilus.desktop", G_FILE_TEST_EXISTS)) {
 		egg_set_desktop_file (DATADIR "/applications/nautilus.desktop");
 	}
@@ -440,6 +472,11 @@ main (int argc, char *argv[])
 	 * happens.
 	 */
 	nautilus_global_preferences_init ();
+
+	/* exit_with_last_window being FALSE, nautilus can run without window. */
+	exit_with_last_window =
+		eel_preferences_get_boolean (NAUTILUS_PREFERENCES_EXIT_WITH_LAST_WINDOW);
+
 	if (no_desktop) {
 		eel_preferences_set_is_invisible
 			(NAUTILUS_PREFERENCES_SHOW_DESKTOP, TRUE);
@@ -497,6 +534,11 @@ main (int argc, char *argv[])
 			 uris);
 		g_strfreev (uris);
 
+		if (unique_app_is_running (application->unique_app) ||
+		    kill_shell) {
+			exit_with_last_window = TRUE;
+		}
+
 		if (is_event_loop_needed ()) {
 			gtk_main ();
 		}
@@ -509,6 +551,8 @@ main (int argc, char *argv[])
 	}
 
  	eel_debug_shut_down ();
+ 	
+ 	nautilus_application_save_accel_map (NULL);
 	
 	return EXIT_SUCCESS;
 }
