@@ -42,39 +42,13 @@
 #include <eel/eel-labeled-image.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
+#include <eel/eel-vfs-extensions.h>
 #include <eel/eel-xml-extensions.h>
-#include <librsvg/rsvg.h>
 #include <libxml/parser.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkcolorseldialog.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkdnd.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtkfilesel.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkselection.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkradiobutton.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkviewport.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
-#include <libgnome/gnome-util.h>
-#include <libgnome/gnome-help.h>
-#include <libgnomeui/gnome-color-picker.h>
-#include <libgnomeui/gnome-icon-entry.h>
-#include <libgnomeui/gnome-help.h>
-#include <libgnomeui/gnome-stock-icons.h>
-#include <libgnomeui/gnome-uidefs.h>
 #include <libnautilus-private/nautilus-customization-data.h>
 #include <libnautilus-private/nautilus-directory.h>
 #include <libnautilus-private/nautilus-emblem-utils.h>
@@ -111,10 +85,8 @@ struct NautilusPropertyBrowserDetails {
 	GtkWidget *bottom_box;
 	
 	GtkWidget *add_button;
-	GtkWidget *add_button_label;
 	GtkWidget *add_button_image;
 	GtkWidget *remove_button;
-	GtkWidget *remove_button_label;
 	GtkWidget *remove_button_image;
 	
 	GtkWidget *patterns_dialog;
@@ -123,7 +95,7 @@ struct NautilusPropertyBrowserDetails {
 	
 	GtkWidget *keyword;
 	GtkWidget *emblem_image;
-	GtkWidget *file_entry;
+	GtkWidget *image_button;
 	
 	GtkWidget *color_picker;
 	GtkWidget *color_name;
@@ -135,6 +107,7 @@ struct NautilusPropertyBrowserDetails {
 	char *dragged_file;
 	char *drag_type;
 	char *image_path;
+	char *filename;
 	
 	NautilusPropertyType category_type;
 	
@@ -186,7 +159,6 @@ static void     emblems_changed_callback                        (GObject
 								 NautilusPropertyBrowser       *property_browser);
 
 /* misc utilities */
-static char *   strip_extension                                 (const char                    *string_to_strip);
 static void     element_clicked_callback                        (GtkWidget                     *image_table,
 								 GtkWidget                     *child,
 								 const EelImageTableEvent *event,
@@ -251,7 +223,7 @@ nautilus_property_browser_init (GtkObject *object)
 {
  	NautilusPropertyBrowser *property_browser;
  	GtkWidget *widget, *temp_box, *temp_hbox, *temp_frame, *vbox;
-	GtkWidget *temp_button, *align;
+	GtkWidget *temp_button;
 	GtkWidget *viewport;
 	char *temp_str;
 	
@@ -278,6 +250,7 @@ nautilus_property_browser_init (GtkObject *object)
 	/* set the title and standard close accelerator */
 	gtk_window_set_title (GTK_WINDOW (widget), _("Backgrounds and Emblems"));
 	gtk_window_set_wmclass (GTK_WINDOW (widget), "property_browser", "Nautilus");
+	gtk_window_set_type_hint (GTK_WINDOW (widget), GDK_WINDOW_TYPE_HINT_DIALOG);
 	eel_gtk_window_set_up_close_accelerator (GTK_WINDOW (widget));
 
 	/* create the main vbox. */
@@ -355,16 +328,11 @@ nautilus_property_browser_init (GtkObject *object)
   	temp_box = gtk_event_box_new();
   	gtk_widget_show(temp_box);
 
-  	temp_frame = gtk_frame_new(NULL);
-  	gtk_frame_set_shadow_type(GTK_FRAME(temp_frame), GTK_SHADOW_NONE);
-  	gtk_widget_show(temp_frame);
-  	gtk_container_add(GTK_CONTAINER(temp_box), temp_frame);
-
   	property_browser->details->bottom_box = gtk_hbox_new (FALSE, 6);
 	gtk_widget_show (property_browser->details->bottom_box);
 	
 	gtk_box_pack_end (GTK_BOX (vbox), temp_box, FALSE, FALSE, 0);
-  	gtk_container_add (GTK_CONTAINER (temp_frame), property_browser->details->bottom_box);
+  	gtk_container_add (GTK_CONTAINER (temp_box), property_browser->details->bottom_box);
   
   	/* create the "help" button */
 	temp_button = gtk_button_new_from_stock (GTK_STOCK_HELP);
@@ -384,17 +352,11 @@ nautilus_property_browser_init (GtkObject *object)
  	g_signal_connect_object (temp_button, "clicked", G_CALLBACK (done_button_callback), property_browser, 0);
 
 	/* create the "remove" button */
-  	property_browser->details->remove_button = gtk_button_new();
-	property_browser->details->remove_button_label = gtk_label_new_with_mnemonic (_("_Remove..."));	
-	gtk_label_set_mnemonic_widget (GTK_LABEL (property_browser->details->remove_button_label),
-				       GTK_WIDGET (property_browser->details->remove_button));
+  	property_browser->details->remove_button = gtk_button_new_with_mnemonic (_("_Remove..."));	
+
 	property_browser->details->remove_button_image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_BUTTON);
-	temp_hbox = gtk_hbox_new (FALSE, 2);
-	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-	gtk_box_pack_start (GTK_BOX (temp_hbox), property_browser->details->remove_button_image, FALSE, FALSE, 0);
-	gtk_box_pack_end (GTK_BOX (temp_hbox), property_browser->details->remove_button_label, FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (property_browser->details->remove_button), align);
-	gtk_container_add (GTK_CONTAINER (align), temp_hbox);
+	gtk_button_set_image (GTK_BUTTON (property_browser->details->remove_button),
+			      property_browser->details->remove_button_image);
 	gtk_widget_show_all (property_browser->details->remove_button);
 
 	gtk_box_pack_end (GTK_BOX (property_browser->details->bottom_box),
@@ -404,18 +366,11 @@ nautilus_property_browser_init (GtkObject *object)
 				 G_CALLBACK (remove_button_callback), property_browser, 0);
 
   	/* now create the "add new" button */
-  	property_browser->details->add_button = gtk_button_new ();
-	property_browser->details->add_button_label = gtk_label_new_with_mnemonic (_("_Add new..."));
-	gtk_label_set_mnemonic_widget (GTK_LABEL (property_browser->details->add_button_label),
-				       GTK_WIDGET (property_browser->details->add_button));
+	property_browser->details->add_button = gtk_button_new_with_mnemonic (_("Add new..."));
 
 	property_browser->details->add_button_image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
-	temp_hbox = gtk_hbox_new (FALSE, 2);
-	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-	gtk_box_pack_start (GTK_BOX (temp_hbox), property_browser->details->add_button_image, FALSE, FALSE, 0);
-	gtk_box_pack_end (GTK_BOX (temp_hbox), property_browser->details->add_button_label, FALSE, FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (property_browser->details->add_button), align);
-	gtk_container_add (GTK_CONTAINER (align), temp_hbox);
+	gtk_button_set_image (GTK_BUTTON (property_browser->details->add_button),
+			      property_browser->details->add_button_image);
 	gtk_widget_show_all (property_browser->details->add_button);
 
 	gtk_box_pack_end (GTK_BOX(property_browser->details->bottom_box),
@@ -643,7 +598,7 @@ nautilus_property_browser_drag_data_get (GtkWidget *widget,
 		is_reset = FALSE;
 		if (strcmp (property_browser->details->drag_type,
 			    "property/keyword") == 0) {
-			char* keyword_str = strip_extension(property_browser->details->dragged_file);
+			char *keyword_str = eel_filename_strip_extension(property_browser->details->dragged_file);
 		        gtk_selection_data_set(selection_data, selection_data->target, 8, keyword_str, strlen(keyword_str));
 			g_free(keyword_str);
 			return;	
@@ -655,7 +610,7 @@ nautilus_property_browser_drag_data_get (GtkWidget *widget,
 			
 			/* handle the "reset" case as an image */
 			if (eel_strcmp (property_browser->details->dragged_file, RESET_IMAGE_NAME) != 0) {
-				eel_gdk_color_parse (property_browser->details->dragged_file, &color);
+				gdk_color_parse (property_browser->details->dragged_file, &color);
 
 				colorArray[0] = color.red;
 				colorArray[1] = color.green;
@@ -717,6 +672,7 @@ ensure_file_is_image (GFile *file)
 {
 	GFileInfo *info;
 	const char *mime_type;
+	gboolean ret;
 
 	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, 0, NULL, NULL);
 	if (info == NULL) {
@@ -728,11 +684,13 @@ ensure_file_is_image (GFile *file)
 		return FALSE;
 	}
 
+	ret = (g_content_type_is_a (mime_type, "image/*") &&
+	       !g_content_type_equals (mime_type, "image/svg") &&
+	       !g_content_type_equals (mime_type, "image/svg+xml"));
+	
 	g_object_unref (info);
-
-	return  g_content_type_is_a (mime_type, "image/*") &&
-		!g_content_type_equals (mime_type, "image/svg") &&
-		!g_content_type_equals (mime_type, "image/svg+xml");
+	
+	return ret;
 }
 
 /* create the appropriate pixbuf for the passed in file */
@@ -757,7 +715,7 @@ make_drag_image (NautilusPropertyBrowser *property_browser, const char* file_nam
 			g_free (image_file_name);
 		} else {
 			icon_name = nautilus_emblem_get_icon_name_from_keyword (file_name);
-			info = nautilus_icon_info_lookup_from_name (file_name, NAUTILUS_ICON_SIZE_STANDARD);
+			info = nautilus_icon_info_lookup_from_name (icon_name, NAUTILUS_ICON_SIZE_STANDARD);
 			pixbuf = nautilus_icon_info_get_pixbuf_at_size (info, NAUTILUS_ICON_SIZE_STANDARD);
 			g_object_unref (info);
 			g_free (icon_name);
@@ -784,13 +742,10 @@ make_drag_image (NautilusPropertyBrowser *property_browser, const char* file_nam
 		g_free (user_directory);	
 	}
 	
-	orig_pixbuf = gdk_pixbuf_new_from_file (image_file_name, NULL);
-	
-	if (orig_pixbuf == NULL) {
-		orig_pixbuf = rsvg_pixbuf_from_file_at_max_size (image_file_name,
-								 MAX_ICON_WIDTH, MAX_ICON_HEIGHT,
-								 NULL);
-	}
+	orig_pixbuf = gdk_pixbuf_new_from_file_at_scale (image_file_name,
+							 MAX_ICON_WIDTH, MAX_ICON_HEIGHT,
+							 TRUE,
+							 NULL);
 	
 	g_free (image_file_name);
 
@@ -807,7 +762,7 @@ make_drag_image (NautilusPropertyBrowser *property_browser, const char* file_nam
 		pixbuf = eel_gdk_pixbuf_scale_down_to_fit (orig_pixbuf, MAX_ICON_WIDTH, MAX_ICON_HEIGHT);
 	}
 
-	gdk_pixbuf_unref (orig_pixbuf);
+	g_object_unref (orig_pixbuf);
 
 	return pixbuf;
 }
@@ -826,11 +781,11 @@ make_color_drag_image (NautilusPropertyBrowser *property_browser, const char *co
 
 	color_square = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, COLOR_SQUARE_SIZE, COLOR_SQUARE_SIZE);
 	
-	eel_gdk_color_parse (color_spec, &color);
+	gdk_color_parse (color_spec, &color);
 	color.red >>= 8;
 	color.green >>= 8;
 	color.blue >>= 8;
-	
+ 
 	pixels = gdk_pixbuf_get_pixels (color_square);
 	stride = gdk_pixbuf_get_rowstride (color_square);
 	
@@ -851,7 +806,7 @@ make_color_drag_image (NautilusPropertyBrowser *property_browser, const char *co
 		ret = nautilus_customization_make_pattern_chit (color_square, 
 								property_browser->details->property_chit,
 								trim_edges, FALSE);
-		gdk_pixbuf_unref (color_square);
+		g_object_unref (color_square);
 	} else {
 		ret = color_square;
 	}
@@ -973,7 +928,7 @@ remove_pattern(NautilusPropertyBrowser *property_browser, const char* pattern_na
 
 	/* delete the pattern from the pattern directory */
 	if (g_unlink (pattern_path) != 0) {
-		char *message = g_strdup_printf (_("Sorry, but pattern %s couldn't be deleted."), pattern_name);
+		char *message = g_strdup_printf (_("Sorry, but pattern %s could not be deleted."), pattern_name);
 		char *detail = _("Check that you have permission to delete the pattern.");
 		eel_show_error_dialog (message, detail, GTK_WINDOW (property_browser));
 		g_free (message);
@@ -987,29 +942,15 @@ remove_pattern(NautilusPropertyBrowser *property_browser, const char* pattern_na
 static void
 remove_emblem (NautilusPropertyBrowser *property_browser, const char* emblem_name)
 {
-	/* build the pathname of the emblem */
-	char *emblem_path;
-	char *user_directory;
-
-	user_directory = nautilus_get_user_directory ();
-
-	emblem_path = g_build_filename (user_directory,
-					"emblems",
-					emblem_name,
-					NULL);
-
-	g_free (user_directory);
-
 	/* delete the emblem from the emblem directory */
-	if (g_unlink (emblem_path) != 0) {
-		char *message = g_strdup_printf (_("Sorry, but emblem %s couldn't be deleted."), emblem_name);
+	if (nautilus_emblem_remove_emblem (emblem_name) == FALSE) {
+		char *message = g_strdup_printf (_("Sorry, but emblem %s could not be deleted."), emblem_name);
 		char *detail = _("Check that you have permission to delete the emblem.");
 		eel_show_error_dialog (message, detail, GTK_WINDOW (property_browser));
 		g_free (message);
 	} else {
 		emit_emblems_changed_signal ();
 	}
-	g_free (emblem_path);
 }
 
 /* handle removing the passed in element */
@@ -1040,13 +981,75 @@ nautilus_property_browser_remove_element (NautilusPropertyBrowser *property_brow
 	}
 }
 
+static void
+update_preview_cb (GtkFileChooser *fc,
+		   GtkImage *preview)
+{
+	char *filename;
+	GdkPixbuf *pixbuf;
+
+	filename = gtk_file_chooser_get_preview_filename (fc);
+	if (filename) {
+		pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+
+		gtk_file_chooser_set_preview_widget_active (fc, pixbuf != NULL);
+		
+		if (pixbuf) {
+			gtk_image_set_from_pixbuf (preview, pixbuf);
+			g_object_unref (pixbuf);
+		}
+		
+		g_free (filename);
+	}
+}
+
+static void
+icon_button_clicked_cb (GtkButton *b,
+			NautilusPropertyBrowser *browser)
+{
+	GtkWidget *dialog;
+	GtkFileFilter *filter;
+	GtkWidget *preview;
+	int res;
+
+	dialog = gtk_file_chooser_dialog_new (_("Select an Image File for the New Emblem"),
+					      GTK_WINDOW (browser),
+					      GTK_FILE_CHOOSER_ACTION_OPEN, 
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+					      NULL);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
+					     DATADIR "/pixmaps");
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_add_pixbuf_formats (filter);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
+
+	preview = gtk_image_new ();
+	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (dialog),
+					     preview);
+	g_signal_connect (dialog, "update-preview",
+			  G_CALLBACK (update_preview_cb), preview);
+
+	res = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	if (res == GTK_RESPONSE_ACCEPT) {
+		/* update the image */
+		g_free (browser->details->filename);
+		browser->details->filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+		gtk_image_set_from_file (GTK_IMAGE (browser->details->image_button), browser->details->filename);
+	}
+
+	gtk_widget_destroy (dialog);
+}
+
 /* here's where we create the emblem dialog */
 static GtkWidget*
 nautilus_emblem_dialog_new (NautilusPropertyBrowser *property_browser)
 {
-	GtkWidget *hbox;
 	GtkWidget *widget;
+	GtkWidget *button;
 	GtkWidget *dialog;
+	GtkWidget *label;
 	GtkWidget *table = gtk_table_new(2, 2, FALSE);
 
 	dialog = gtk_dialog_new_with_buttons (_("Create a New Emblem"),
@@ -1086,31 +1089,29 @@ nautilus_emblem_dialog_new (NautilusPropertyBrowser *property_browser)
 				       GTK_WIDGET (property_browser->details->keyword));
 
 	/* default image is the generic emblem */
-	g_free(property_browser->details->image_path);		
+	g_free (property_browser->details->image_path);		
 	property_browser->details->image_path = g_build_filename (NAUTILUS_PIXMAPDIR, "emblems.png", NULL);
 	
-	/* set up a gnome icon entry to pick the image file */
-	widget = gtk_label_new_with_mnemonic (_("_Image:"));
-	gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
-	gtk_widget_show(widget);
-	gtk_table_attach(GTK_TABLE(table), widget, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+	/* set up a file chooser to pick the image file */
+	label = gtk_label_new_with_mnemonic (_("_Image:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_widget_show (label);
+	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox);
+	widget = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (widget);
 
-	property_browser->details->file_entry = gnome_icon_entry_new (NULL, _("Select an Image File for the New Emblem"));
-	gnome_icon_entry_set_pixmap_subdir (GNOME_ICON_ENTRY(property_browser->details->file_entry),
-						DATADIR "/pixmaps");
-	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY(property_browser->details->file_entry),
-				       property_browser->details->image_path);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (widget),
-				       GTK_WIDGET (property_browser->details->file_entry));
+	button = gtk_button_new ();
+	property_browser->details->image_button = gtk_image_new_from_file (property_browser->details->image_path);
+	gtk_button_set_image (GTK_BUTTON (button), property_browser->details->image_button);
+	g_signal_connect (button, "clicked", G_CALLBACK (icon_button_clicked_cb),
+			  property_browser);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
+
+	gtk_widget_show (button);
+	gtk_table_attach (GTK_TABLE (table), widget, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_box_pack_start (GTK_BOX (widget), button, FALSE, FALSE, 0);
 
-	gtk_widget_show(property_browser->details->file_entry);
-	gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (property_browser->details->file_entry), FALSE, FALSE, 0);
-	
-	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (property_browser->details->file_entry), property_browser->details->image_path);	
 	return dialog;
 }
 
@@ -1156,93 +1157,79 @@ nautilus_color_selection_dialog_new (NautilusPropertyBrowser *property_browser)
 	widget = gtk_label_new_with_mnemonic(_("Color _value:"));
 	gtk_widget_show(widget);
 	gtk_table_attach(GTK_TABLE(table), widget, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
- 
-	/* set up a gnome file entry to pick the image file */
-	property_browser->details->color_picker = gnome_color_picker_new ();
+
+	property_browser->details->color_picker = gtk_color_button_new ();
 	gtk_widget_show (property_browser->details->color_picker);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (widget), property_browser->details->color_picker);
 
 	gtk_table_attach(GTK_TABLE(table), property_browser->details->color_picker, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-		
-	
+
 	return dialog;
 }
 
 /* add the newly selected file to the browser images */
 static void
-add_pattern_to_browser (const char *path_name, gpointer *data)
+add_pattern_to_browser (GtkDialog *dialog, gint response_id, gpointer *data)
 {
-	char *directory_path, *source_file_name, *destination_name;
-	char *path_uri, *basename;
+	char *directory_path, *destination_name;
+	char *basename;
 	char *user_directory;	
-	GFile *src, *dest;
+	GFile *dest, *selected;
 	
 	NautilusPropertyBrowser *property_browser = NAUTILUS_PROPERTY_BROWSER (data);
 
-	/* make sure that it's a valid path */
-	if (path_name == NULL || path_name[0] != '/') {
-		char *message;
-		char *detail;
-		if (path_name != NULL) {
-			message = g_strdup_printf (_("Sorry, but \"%s\" is not a valid file name."), path_name);
-			detail = _("Please check the spelling and try again.");
-		} else {
-			message = g_strdup (_("Sorry, but you did not supply a valid file name."));
-			detail = _("Please try again.");
-		}
-		eel_show_error_dialog (message, detail, GTK_WINDOW (property_browser));
-		g_free (message);
+	if (response_id != GTK_RESPONSE_ACCEPT) {
+		gtk_widget_hide (GTK_WIDGET (dialog));
 		return;
 	}
 	
-	/* fetch the mime type and make sure that the file is an image */
-	path_uri = g_filename_to_uri (path_name, NULL, NULL);	
-
+	selected = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
+	
 	/* don't allow the user to change the reset image */
-	basename = g_path_get_basename (path_name);
+	basename = g_file_get_basename (selected);
 	if (basename && eel_strcmp (basename, RESET_IMAGE_NAME) == 0) {
-		eel_show_error_dialog (_("Sorry, but you can't replace the reset image."), 
+		eel_show_error_dialog (_("Sorry, but you cannot replace the reset image."), 
 		                       _("Reset is a special image that cannot be deleted."), 
 		                       NULL);
-		g_free (path_uri);
+		g_object_unref (selected);
 		g_free (basename);
 		return;
 	}
 		
-	g_free (path_uri);	
-	g_free (basename);
 		
 	user_directory = nautilus_get_user_directory ();
 		
 	/* copy the image file to the patterns directory */
 	directory_path = g_build_filename (user_directory, "patterns", NULL);
 	g_free (user_directory);
-	source_file_name = strrchr (path_name, '/');
-	destination_name = g_build_filename (directory_path, source_file_name + 1, NULL);
+	destination_name = g_build_filename (directory_path, basename, NULL);
 
 	/* make the directory if it doesn't exist */
 	if (!g_file_test (directory_path, G_FILE_TEST_EXISTS)) {
 		g_mkdir_with_parents (directory_path, 0775);
 	}
 		
+	dest = g_file_new_for_path (destination_name);
+	
+	g_free (destination_name);
 	g_free (directory_path);
 
-	src = g_file_new_for_path (path_name);
-	dest = g_file_new_for_path (destination_name);
-	if (!g_file_copy (src, dest,
+	if (!g_file_copy (selected, dest,
 			  0,
 			  NULL, NULL, NULL, NULL)) {
-		char *message = g_strdup_printf (_("Sorry, but the pattern %s couldn't be installed."), path_name);
+		char *message = g_strdup_printf (_("Sorry, but the pattern %s could not be installed."), basename);
 		eel_show_error_dialog (message, NULL, GTK_WINDOW (property_browser));
 		g_free (message);
 	}
-	g_object_unref (src);
+	g_object_unref (selected);
 	g_object_unref (dest);
+	g_free (basename);
 	
-	g_free (destination_name);
 	
 	/* update the property browser's contents to show the new one */
 	nautilus_property_browser_update_contents (property_browser);
+
+	gtk_widget_hide (GTK_WIDGET (dialog));
 }
 
 /* here's where we initiate adding a new pattern by putting up a file selector */
@@ -1250,16 +1237,40 @@ add_pattern_to_browser (const char *path_name, gpointer *data)
 static void
 add_new_pattern (NautilusPropertyBrowser *property_browser)
 {
+	GtkWidget *dialog;
+	GtkFileFilter *filter;
+	GtkWidget *preview;
+	
 	if (property_browser->details->patterns_dialog) {
 		gtk_window_present (GTK_WINDOW (property_browser->details->patterns_dialog));
 	} else {
-		property_browser->details->patterns_dialog = 
-			eel_gnome_icon_selector_new (_("Select an Image File to Add as a Pattern"),
-				DATADIR "/nautilus/patterns/",
-				GTK_WINDOW (property_browser),
-				(EelIconSelectionFunction) add_pattern_to_browser,
-				property_browser);						   
+		property_browser->details->patterns_dialog = dialog = 
+			gtk_file_chooser_dialog_new (_("Select an Image File to Add as a Pattern"),
+						     GTK_WINDOW (property_browser),
+						     GTK_FILE_CHOOSER_ACTION_OPEN, 
+						     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+						     NULL);
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
+						     DATADIR "/nautilus/patterns/");
+		filter = gtk_file_filter_new ();
+		gtk_file_filter_add_pixbuf_formats (filter);
+		gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
 
+		gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
+		
+		preview = gtk_image_new ();
+		gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (dialog),
+						     preview);
+		g_signal_connect (dialog, "update-preview",
+				  G_CALLBACK (update_preview_cb), preview);
+		
+		g_signal_connect (dialog, "response",
+				  G_CALLBACK (add_pattern_to_browser),
+				  property_browser);
+
+		gtk_widget_show (GTK_WIDGET (dialog));
+		
 		if (property_browser->details->patterns_dialog)
 			eel_add_weak_pointer (&property_browser->details->patterns_dialog);
 	}
@@ -1319,20 +1330,17 @@ add_color_to_file (NautilusPropertyBrowser *property_browser, const char *color_
 static void
 add_color_to_browser (GtkWidget *widget, gint which_button, gpointer *data)
 {
-	char color_spec[8];
+	char * color_spec;
 	const char *color_name;
 	char *stripped_color_name;
 
-	gdouble color[4];
 	NautilusPropertyBrowser *property_browser = NAUTILUS_PROPERTY_BROWSER (data);
 
 	if (which_button == GTK_RESPONSE_OK) {
-		gnome_color_picker_get_d (GNOME_COLOR_PICKER (property_browser->details->color_picker), &color[0], &color[1], &color[2], &color[3]);		
-		g_snprintf (color_spec, sizeof (color_spec),
-			 "#%02X%02X%02X",
-			 (guint) (color[0] * 255.0 + 0.5),
-			 (guint) (color[1] * 255.0 + 0.5),
-			 (guint) (color[2] * 255.0 + 0.5));
+		GdkColor color;
+
+		gtk_color_button_get_color (GTK_COLOR_BUTTON (property_browser->details->color_picker), &color);
+		color_spec = gdk_color_to_string (&color);
 
 		color_name = gtk_entry_get_text (GTK_ENTRY (property_browser->details->color_name));
 		stripped_color_name = g_strstrip (g_strdup (color_name));
@@ -1346,6 +1354,7 @@ add_color_to_browser (GtkWidget *widget, gint which_button, gpointer *data)
 			nautilus_property_browser_update_contents(property_browser);
 		}
 		g_free (stripped_color_name);
+		g_free (color_spec);
 	}
 	
 	gtk_widget_destroy(property_browser->details->colors_dialog);
@@ -1366,17 +1375,16 @@ show_color_selection_window (GtkWidget *widget, gpointer *data)
 	property_browser->details->colors_dialog = nautilus_color_selection_dialog_new (property_browser);		
 
 	/* set the color to the one picked by the selector */
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (property_browser->details->color_picker), color.red, color.green, color.blue, 1.0);
-	
+	gtk_color_button_set_color (GTK_COLOR_BUTTON (property_browser->details->color_picker), &color);
+
 	/* connect the signals to the new dialog */
-	
+
 	eel_add_weak_pointer (&property_browser->details->colors_dialog);
 
 	g_signal_connect_object (property_browser->details->colors_dialog, "response",
 				 G_CALLBACK (add_color_to_browser), property_browser, 0);
 	gtk_window_set_position (GTK_WINDOW (property_browser->details->colors_dialog), GTK_WIN_POS_MOUSE);
 	gtk_widget_show (GTK_WIDGET(property_browser->details->colors_dialog));
-
 }
 
 
@@ -1419,24 +1427,23 @@ emblem_dialog_clicked (GtkWidget *dialog, int which_button, NautilusPropertyBrow
 	if (which_button == GTK_RESPONSE_OK) {
 
 		/* update the image path from the file entry */
-		if (property_browser->details->file_entry) {
-			emblem_path = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (property_browser->details->file_entry));
-			if (emblem_path) {
-				emblem_file = g_file_new_for_path (emblem_path);
-				if (ensure_file_is_image (emblem_file)) {
-					g_free (property_browser->details->image_path);
-					property_browser->details->image_path = emblem_path;				
-				} else {
-					char *message = g_strdup_printf
-						(_("Sorry, but \"%s\" is not a usable image file."), emblem_path);
-					eel_show_error_dialog (_("The file is not an image."), message, GTK_WINDOW (property_browser));
-					g_free (message);
-					g_free (emblem_path);
-					g_object_unref (emblem_file);
-					return;
-				}
+		if (property_browser->details->filename) {
+			emblem_path = property_browser->details->filename;
+			emblem_file = g_file_new_for_path (emblem_path);
+			if (ensure_file_is_image (emblem_file)) {
+				g_free (property_browser->details->image_path);
+				property_browser->details->image_path = emblem_path;
+			} else {
+				char *message = g_strdup_printf
+					(_("Sorry, but \"%s\" is not a usable image file."), emblem_path);
+				eel_show_error_dialog (_("The file is not an image."), message, GTK_WINDOW (property_browser));
+				g_free (message);
+				g_free (emblem_path);
+				emblem_path = NULL;
 				g_object_unref (emblem_file);
+				return;
 			}
+			g_object_unref (emblem_file);
 		}
 
 		emblem_file = g_file_new_for_path (property_browser->details->image_path);
@@ -1476,7 +1483,7 @@ emblem_dialog_clicked (GtkWidget *dialog, int which_button, NautilusPropertyBrow
 	
 	property_browser->details->keyword = NULL;
 	property_browser->details->emblem_image = NULL;
-	property_browser->details->file_entry = NULL;
+	property_browser->details->filename = NULL;
 }
 
 /* here's the routine to add a new emblem, by putting up an emblem dialog */
@@ -1550,9 +1557,9 @@ help_button_callback (GtkWidget *widget, GtkWidget *property_browser)
 	GError *error = NULL;
 	GtkWidget *dialog;
 
-	gnome_help_display_desktop_on_screen (
-		NULL, "user-guide", "user-guide.xml", "gosnautilus-50",
-		gtk_window_get_screen (GTK_WINDOW (property_browser)), &error);
+	gtk_show_uri (gtk_widget_get_screen (property_browser),
+		      "ghelp:user-guide#gosnautilus-50",
+		      gtk_get_current_event_time (), &error);
 
 	if (error) {
 		dialog = gtk_message_dialog_new (GTK_WINDOW (property_browser),
@@ -1644,22 +1651,6 @@ element_clicked_callback (GtkWidget *image_table,
 	}
 }
 
-
-/* utility routine to strip the extension from the passed in string */
-static char*
-strip_extension (const char* string_to_strip)
-{
-	char *result_str, *temp_str;
-	if (string_to_strip == NULL)
-		return NULL;
-	
-	result_str = g_strdup(string_to_strip);
-	temp_str = strrchr(result_str, '.');
-	if (temp_str)
-		*temp_str = '\0';
-	return result_str;
-}
-
 static void
 labeled_image_configure (EelLabeledImage *labeled_image)
 {
@@ -1702,7 +1693,6 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 	GList *icons, *l;
 	char *icon_name;
 	char *keyword;
-	char *extension;
 	GtkWidget *property_image;
 	GtkWidget *blank;
 	guint num_images;
@@ -1720,6 +1710,7 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 
 		icons = nautilus_emblem_list_available ();
 
+		property_browser->details->has_local = FALSE;
 		l = icons;
 		while (l != NULL) {
 			icon_name = (char *)l->data;
@@ -1728,8 +1719,15 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 			if (!nautilus_emblem_should_show_in_list (icon_name)) {
 				continue;
 			}
+
 			object_name = nautilus_emblem_get_keyword_from_icon_name (icon_name);
-			info = nautilus_icon_info_lookup_from_name (object_name, NAUTILUS_ICON_SIZE_STANDARD);
+			if (nautilus_emblem_can_remove_emblem (object_name)) {
+				property_browser->details->has_local = TRUE;
+			} else if (property_browser->details->remove_mode) {
+				g_free (object_name);
+				continue;
+			}
+			info = nautilus_icon_info_lookup_from_name (icon_name, NAUTILUS_ICON_SIZE_STANDARD);
 			object_pixbuf = nautilus_icon_info_get_pixbuf_at_size (info, NAUTILUS_ICON_SIZE_STANDARD);
 			object_label = g_strdup (nautilus_icon_info_get_display_name (info));
 			g_object_unref (info);
@@ -1741,11 +1739,7 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 			property_image = labeled_image_new (object_label, object_pixbuf, object_name, PANGO_SCALE_LARGE);
 			eel_labeled_image_set_fixed_image_height (EEL_LABELED_IMAGE (property_image), MAX_EMBLEM_HEIGHT);
 
-			keyword = g_strdup (object_name);
-			extension = strchr (keyword, '.');
-			if (extension) {
-				*extension = '\0';
-			}
+			keyword = eel_filename_strip_extension (object_name);
 			property_browser->details->keywords = g_list_prepend (property_browser->details->keywords,
 									      keyword);
 
@@ -1759,7 +1753,6 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 			}
 		}
 		eel_g_list_free_deep (icons);
-		property_browser->details->has_local = FALSE;
 	} else {
 		customization_data = nautilus_customization_data_new (property_browser->details->category,
 								      !property_browser->details->remove_mode,
@@ -2187,7 +2180,7 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 					  GTK_ICON_SIZE_BUTTON);
 		
 		if (text != NULL) {
-			gtk_label_set_text_with_mnemonic (GTK_LABEL(property_browser->details->add_button_label), text);
+			gtk_button_set_label (GTK_BUTTON (property_browser->details->add_button), text);
 
 		}
 		gtk_widget_show (property_browser->details->add_button);
@@ -2257,7 +2250,7 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 		else
 			gtk_widget_show(property_browser->details->remove_button);
 		if (text != NULL) {
-			gtk_label_set_text_with_mnemonic (GTK_LABEL(property_browser->details->remove_button_label), text);
+			gtk_button_set_label (GTK_BUTTON (property_browser->details->remove_button), text);
 		}
 	}
 }
